<?php

namespace InspireLabs\WooInpost\shipping;

use InspireLabs\WooInpost\EasyPack;
use Exception;
use InspireLabs\WooInpost\EasyPack_API;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Cod_Model;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Contstants;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Model;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Parcel_Dimensions_Model;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Parcel_Model;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Parcel_Weight_Model;
use ReflectionException;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'EasyPack_Shipping_Method_Courier_Local_Standard_COD' ) ) {
	class EasyPack_Shipping_Method_Courier_Local_Standard_COD
		extends EasyPack_Shippng_Parcel_Machines {

		const WP_AJAX_ACTION_CREATE = 'courier_local_standard_cod_create_package';

		const NONCE_ACTION = self::SERVICE_ID;

		const SERVICE_ID = ShipX_Shipment_Model::SERVICE_INPOST_COURIER_LOCAL_STANDARD;

		/**
		 * Constructor for shipping class
		 *
		 * @access public
		 * @return void
		 */
		public function __construct( $instance_id = 0 ) {
			$this->init_form_fields();
			$this->instance_id  = absint( $instance_id );
			$this->supports     = [
				'shipping-zones',
				'instance-settings',
			];
			$this->id           = 'easypack_shipping_courier_local_standard_cod';
			$this->method_title = __( 'InPost Courier Local Standard COD',
				EasyPack::$text_domain );
			$this->init();
		}

		public function generate_rates_html( $key, $data ) {
			$rates = get_option( 'woocommerce_' . $this->id . '_rates', [] );
			ob_start();
			include( 'views/html-rates-courier.php' );

			return ob_get_clean();
		}

		public function init_form_fields() {
			$bank_accounts = [];
			//$bacs = new WC_Gateway_BACS();
			//$accounts = $bacs->account_details;
			$accounts = get_option( 'woocommerce_bacs_accounts', [] );
			foreach ( $accounts as $account ) {
				if ( isset( $account['iban'] ) && $account['iban'] != '' ) {
					$bank_accounts[ $account['iban'] ] = $account['iban'] . ' '
					                                     . $account['bank_name'];
				}
			}

			$settings                   = [
				[
					'title'       => __( 'General settings',
						EasyPack::$text_domain ),
					'type'        => 'title',
					'description' => '',
					'id'          => 'section_general_settings',
				],
				'logo_upload'        => [
					'name'  => __( 'Change logo', '' ),
					'title' => __( 'Upload custom logo',
						EasyPack::$text_domain ),
					'type'  => 'logo_upload',
					'id'    => 'logo_upload',
				],
				'title'              => [
					'title'             => __( 'Method title',
						EasyPack::$text_domain ),
					'type'              => 'text',
					'default'           => __( 'InPost Courier COD',
						EasyPack::$text_domain ),
					'custom_attributes' => [ 'required' => 'required' ],
					'desc_tip'          => false,
				],
				/* RM: 11101
									'bank_account' => array(
											'title' 			=> __( 'Bank account', EasyPack::$text_domain ),
											'type' 				=> 'select',
											'desc_tip'	 		=> __( 'Select the account to which the cost of COD will be transferred. If the list is empty, add the account number in the Settings.', EasyPack::$text_domain ),
											'class'    			=> 'wc-enhanced-select1',
											'custom_attributes' => array( 'required' => 'required' ),
											'options' 			=> $bank_accounts
									),
				*/
				'free_shipping_cost' => [
					'title'             => __( 'Free shipping',
						EasyPack::$text_domain ),
					'type'              => 'number',
					'custom_attributes' => [ 'step' => 'any', 'min' => '0' ],
					'default'           => '',
					'desc_tip'          => __( 'Enter the amount of the contract, from which shipping will be free (does not include virtual products).',
						EasyPack::$text_domain ),
					'placeholder'       => '0.00',
				],
				'flat_rate'          => [
					'title'   => __( 'Flat rate', EasyPack::$text_domain ),
					'type'    => 'checkbox',
					'label'   => __( 'Set a flat-rate shipping fee for the entire order.',
						EasyPack::$text_domain ),
					'class'   => 'easypack_flat_rate',
					'default' => 'yes',
				],
				'cost_per_order'     => [
					'title'             => __( 'Cost per order',
						EasyPack::$text_domain ),
					'type'              => 'number',
					'custom_attributes' => [ 'step' => 'any', 'min' => '0' ],
					'class'             => 'easypack_cost_per_order',
					'default'           => '',
					'desc_tip'          => __( 'Set a flat-rate shipping for all orders'
						, EasyPack::$text_domain ),
					'placeholder'       => '0.00',
				],

				'tax_status' => [
					'title'   => __( 'Tax status', 'woocommerce' ),
					'type'    => 'select',
					'class'   => 'wc-enhanced-select',
					'default' => 'none',
					'options' => [
						'none'    => _x( 'None', 'Tax status',
							EasyPack::$text_domain ),
						'taxable' => __( 'Taxable', EasyPack::$text_domain ),
					],
				],

				/*[
					'title' => __('Rates table', EasyPack::$text_domain),
					'type' => 'title',
					'description' => '',
					'id' => 'section_general_settings',
				],*/

				'based_on' => [
					'title'    => __( 'Based on', EasyPack::$text_domain ),
					'type'     => 'select',
					'desc_tip' => __( 'Select the method of calculating shipping cost. If the cost of shipping is to be calculated based on the weight of the cart and the products do not have a defined weight, the cost will be calculated incorrectly.',
						EasyPack::$text_domain ),
					'class'    => 'wc-enhanced-select easypack_based_on',
					'options'  => [
						'price'  => __( 'Price', EasyPack::$text_domain ),
						'weight' => __( 'Weight', EasyPack::$text_domain ),
					],
				],
				'rates'    => [
					'title'    => '',
					'type'     => 'rates',
					'class'    => 'easypack_rates',
					'default'  => '',
					'desc_tip' => '',
				],

				'front_display_order' => [
					'title'    => __( 'Front display order',
						EasyPack::$text_domain ),
					'type'     => 'number',
					'default'  => __( '1',
						EasyPack::$text_domain ),
					'desc_tip' => false,
				],

			];
			$this->form_fields          = $settings;
			$this->instance_form_fields = $settings;
		}

		public function process_admin_options() {
			parent::process_admin_options();
			EasyPack_API()->clear_cache();

			/* RM 11101
						$args = array();
						$args['iban'] = $this->get_option( 'bank_account' );

						try {
							EasyPack_API()->update_customer( $args );
						}
						catch ( Exception $e ) {
							WC_Admin_Settings::add_error( strip_tags( $e->getMessage() ) );
						}
			*/

		}


		public function save_post( $post_id ) {
			// Check if our nonce is set.
			if ( ! isset( $_POST['wp_nonce'] ) ) {
				return;
			}
			// Verify that the nonce is valid.
			if ( ! wp_verify_nonce( $_POST['wp_nonce'], self::NONCE_ACTION ) ) {
				return;
			}
			// If this is an autosave, our form has not been submitted, so we don't want to do anything.
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			$status = get_post_meta( $post_id, '_easypack_status', true );
			if ( $status == '' ) {
				$status = 'new';
			}

			if ( $status == 'new' ) {
				$parcel_machine_id   = $_POST['parcel_machine_id'];
				$parcel_machine_desc = $_POST['parcel_machine_desc'];

				update_post_meta( $post_id, '_parcel_machine_id',
					$parcel_machine_id );
				update_post_meta( $post_id, '_parcel_machine_desc',
					$parcel_machine_desc );


				$parcels         = $_POST['parcel'];
				$cod_amounts     = $_POST['cod_amount'];
				$easypack_pacels = [];
				foreach ( $parcels as $key => $parcel ) {
					$easypack_pacels[] = [
						'package_size' => $parcel,
						'cod_amount'   => $cod_amounts[ $key ],
					];
				}
				update_post_meta( $post_id, '_easypack_parcels',
					$easypack_pacels );

				$easypack_send_method = $_POST['easypack_send_method'];
				update_post_meta( $post_id, '_easypack_send_method',
					$easypack_send_method );
			}

		}

		public function add_meta_boxes( $post_type, $post ) {
			if ( $post->post_type == 'shop_order' ) {
				$order = wc_get_order( $post->ID );
				if ( $order->has_shipping_method( $this->id ) ) {
					add_meta_box( 'easypack_parcel_machines',
						__( 'InPost',
							EasyPack::$text_domain ) . $this->get_logo(),
						[ $this, 'order_metabox' ], 'shop_order', 'side',
						'default' );
				}
			}
		}

		public function order_metabox( $post ) {
			self::order_metabox_content( $post );
		}


		public function woocommerce_review_order_after_shipping() {

			/*if ( in_array( $this->id, WC()->session->get('chosen_shipping_methods') ) ) {
				wc_get_template(
					'checkout/easypack-review-order-after-shipping-courier.php'
					, []
					, ''
					, plugin_dir_path(
						EasyPack()->getPluginFilePath()
					)
					. 'templates/' );
			}*/
		}

		/**
		 * @param unknown $package
		 *
		 */
		public function calculate_shipping_table_rate( $package ) {
			$rates = get_option( 'woocommerce_' . $this->id . '_rates', [] );
			foreach ( $rates as $key => $rate ) {
				if ( empty( $rates[ $key ]['min'] )
				     || trim( $rates[ $key ]['min'] ) == ''
				) {
					$rates[ $key ]['min'] = 0;
				}
				if ( empty( $rates[ $key ]['max'] )
				     || trim( $rates[ $key ]['max'] ) == ''
				) {
					$rates[ $key ]['max'] = PHP_INT_MAX;
				}
			}
			$value = 0;
			if ( $this->based_on == 'price' ) {
				$value = $this->package_subtotal( $package['contents'] );
			}
			if ( $this->based_on == 'weight' ) {
				$value = $this->package_weight( $package['contents'] );
			}
			foreach ( $rates as $rate ) {
				if ( floatval( $rate['min'] ) <= $value
				     && floatval( $rate['max'] ) >= $value
				) {
					$cost = 0;
					if ( isset( $rate['percent'] )
					     && floatval( $rate['percent'] ) != 0
					) {
						$cost = $package['contents_cost']
						        * ( floatval( $rate['percent'] ) / 100 );
					}
					$cost     = $cost + floatval( $rate['cost'] );
					$add_rate = [
						'id'    => $this->id,
						'label' => $this->title,
						'cost'  => $cost,
					];
					$this->add_rate( $add_rate );

					return;
				}
			}
		}

		/**
		 * @return ShipX_Shipment_Model
		 */
		public static function ajax_create_shipment_model() {
			$shipmentService = EasyPack::EasyPack()->get_shipment_service();
			$cod_amounts     = isset( $_POST['cod_amounts'] ) ? $_POST['cod_amounts']
				: null;

			if ( ! isset( $_POST['insurance_amounts'] ) || $_POST['insurance_amounts'][0] === '0' ) {
				$insurance_amounts = [ null ];
			} else {
				$insurance_amounts = $_POST['insurance_amounts'];
			}

			$shipment
				= $shipmentService->create_shipment_object_by_shiping_data(
				$_POST['parcels'],
				(int) $_POST['order_id'],
				$_POST['send_method'],
				self::SERVICE_ID,
				[
					'length'       => $_POST['parcel_length'],
					'width'        => $_POST['parcel_width'],
					'height'       => $_POST['parcel_height'],
					'weight'       => $_POST['parcel_weight'],
					'non_standard' => $_POST['parcel_non_standard'],
				],
				null,
				$cod_amounts[0],
				$insurance_amounts[0],
				$_POST['reference_number']
			);
			$shipment->getInternalData()
			         ->setOrderId( (int) $_POST['order_id'] );

			return $shipment;
		}

		/**
		 * @param bool $courier
		 *
		 * @throws ReflectionException
		 */
		public static function ajax_create_package( $courier = false ) {
			$ret = [ 'status' => 'ok' ];

			$shipment_model   = self::ajax_create_shipment_model();
			$order_id         = $shipment_model->getInternalData()
			                                   ->getOrderId();
			$shipment_service = EasyPack::EasyPack()->get_shipment_service();
			$shipment_array
			                  = $shipment_service->shipment_to_array( $shipment_model );
			$status_service   = EasyPack::EasyPack()
			                            ->get_shipment_status_service();
			$label_url        = '';

			$order       = wc_get_order( (int) $_POST['order_id'] );
			$cod_amounts = isset( $_POST['cod_amounts'] ) ? $_POST['cod_amounts']
				: null;
			if ( is_array( $cod_amounts ) ) {
				$total_amount = 0;
				foreach ( $cod_amounts as $amount ) {
					$total_amount = $total_amount + floatval( $amount );
				}

				if ( $total_amount != $order->get_total() ) {
					$ret['status'] = 'error';
					$ret['message']
					               = sprintf( __( 'Order total %s do not equals total COD amounts %s.',
						EasyPack::$text_domain ),
						$order->get_total(), $total_amount );
				}

				$cod = new ShipX_Shipment_Cod_Model();
				$cod->setCurrency( ShipX_Shipment_Contstants::CURRENCY_PLN );
				$cod->setAmount( $total_amount );
				$shipment_model->setCod( $cod );
			} else {
				$cod = new ShipX_Shipment_Cod_Model();
				$cod->setCurrency( ShipX_Shipment_Contstants::CURRENCY_PLN );
				$cod->setAmount( $order->get_total() );
				$shipment_model->setCod( $cod );
			}


			try {
				update_post_meta( $shipment_model->getInternalData()
				                                 ->getOrderId(),
					'_easypack_parcel_create_args',
					$shipment_array );
				$response
					= EasyPack_API()->customer_parcel_create( $shipment_array );

				$internal_data = $shipment_model->getInternalData();
				$internal_data->setInpostId( $response['id'] );
				$internal_data->setStatus( $response['status'] );
				$internal_data->setStatusTitle(
					$status_service->getStatusTitle( $response['status'] )
				);
				$internal_data->setStatusDescription(
					$status_service->getStatusDescription( $response['status'] )
				);
				$internal_data->setStatusChangedTimestamp( time() );
				$internal_data->setCreatedAt( time() );
				$internal_data->setUrl( $response['href'] );
				$shipment_model->setInternalData( $internal_data );
				$shipment_service->update_shipment_to_db( $shipment_model );

				$search_in_api
					= EasyPack_API()->customer_parcel_get_by_id( $shipment_model->getInternalData()
					                                                            ->getInpostId() );

				$label_url = null;
				//if ('courier' === $_POST['send_method']) {
				for ( $i = 0; $i < 10; $i ++ ) {
					sleep( 1 );
					$label_url
						= self::ajax_parcel_machines_get_stickers_url( $shipment_model );
					if ( ! empty( $label_url ) ) {
						break;
					}
					/*
					 * {"status":400,"error":"invalid_action","message":"Action (get_label) can not be taken on shipment with status (created).","details":{"action":"get_label","shipment_status":"created","shipment_id":53256}}
					 */
				}
				//}

				for ( $i = 0; $i < 3; $i ++ ) {
					sleep( 1 );
					$search_in_api
						= EasyPack_API()->customer_parcel_get_by_id( $shipment_model->getInternalData()
						                                                            ->getInpostId() );
					if ( isset( $search_in_api['items'][0]['tracking_number'] ) ) {
						$shipment_model->getInternalData()->setTrackingNumber(
							$search_in_api['items'][0]['tracking_number']
						);
						break;
					}
				}

				$internal_data = $shipment_model->getInternalData();
				$internal_data->setLabelUrl( $label_url );
				$shipment_model->setInternalData( $internal_data );
				$shipment_service->update_shipment_to_db( $shipment_model );
				update_post_meta( $order_id, '_easypack_status', 'created' );

				//zapisz koszt przesyłki do przesyłki
				$price_calculator
					= EasyPack()->get_shipment_price_calculator_service();

				//$shipment_cost = $price_calculator->get_calculated_shipment_cost($shipment_model);
				//$shipment_model->getInternalData()->setShipmentCost($shipment_cost);

				$shipment_service->update_shipment_to_db( $shipment_model );
			} catch ( Exception $e ) {
				$ret['status'] = 'error';
				$ret['message']
				               = __( 'There are some errors. Please fix it: <br>'
						, EasyPack::$text_domain )
				                 . $e->getMessage();
			}

			if ( $ret['status'] == 'ok' ) {
				wc_get_order( $order_id )->add_order_note(
					__( 'Shipment created', EasyPack::$text_domain ), false
				);
				$ret['content']
					= self::order_metabox_content( get_post( $order_id ), false,
					$shipment_model );
			}
			echo json_encode( $ret );
			wp_die();
		}


		/**
		 * @param                           $post
		 * @param bool $output
		 *
		 * @param ShipX_Shipment_Model|null $shipment
		 *
		 * @return string
		 */
		public static function order_metabox_content(
			$post,
			$output = true,
			$shipment = null
		) {
			$wp_ajax_action_create = self::WP_AJAX_ACTION_CREATE;
			if ( ! $output ) {
				ob_start();
			}
			$shipment_service = EasyPack::EasyPack()->get_shipment_service();
			$order_id         = $post->ID;
			if ( false === $shipment instanceof ShipX_Shipment_Model ) {
				$shipment
					= $shipment_service->get_shipment_by_order_id( $order_id );
			}

			if ( $shipment instanceof ShipX_Shipment_Model
			     && false === $shipment_service
					->is_shipment_match_to_current_api( $shipment )
			) {
				wp_nonce_field( self::NONCE_ACTION,
					'wp_nonce' );
				$wrong_api_env = true;
				include( 'views/html-order-metabox-courier-cod.php' );
				if ( ! $output ) {
					$out = ob_get_clean();

					return $out;
				}

				return '';
			}
			$wrong_api_env = false;

			$order = wc_get_order( $order_id );
			//$parcel_machines = EasyPack_API()->machines_cod_options();

			if ( null !== $shipment ) {
				$parcels      = $shipment->getParcels();
				$tracking_url = $shipment->getInternalData()
				                         ->getTrackingNumber();
				$stickers_url = $shipment->getInternalData()->getLabelUrl();

				if ( true === $output ) {
					$status_srv = EasyPack()->get_shipment_status_service();
					$status_srv->refreshStatus( $shipment );
				}

				$parcel_machine_id = $shipment->getCustomAttributes()
				                              ->getTargetPoint();
				$send_method       = $shipment->getCustomAttributes()
				                              ->getSendingMethod();
				$disabled          = true;
			} else {
				$package_sizes_display
					        = EasyPack()->get_package_sizes_display();
				$parcels    = [];
				$parcel     = new ShipX_Shipment_Parcel_Model();
				$dimensions = self::get_single_product_dimensions( $order_id );
				$parcel->setDimensions( $dimensions );
				$weight = new ShipX_Shipment_Parcel_Weight_Model();
				$parcel->setWeight( $weight );
				$parcel->setTemplate( get_option( 'easypack_default_package_size',
					'small' ) );
				$parcels[] = $parcel;

				$parcel_machine_from_order
					               = $order->get_meta( '_parcel_machine_id',
					$order_id );
				$parcel_machine_id = ! empty( $parcel_machine_from_order )
					? $parcel_machine_from_order
					: get_option( 'easypack_default_machine_id' );

				$tracking_url = false;
				$status       = 'new';
				$send_method  = get_option( 'easypack_default_send_method',
					'parcel_machine' );
				$disabled     = false;
			}
			$package_sizes = EasyPack()->get_package_sizes();

			$send_method_disabled = false;
			if ( EasyPack_API()->getCountry() === EasyPack_API::COUNTRY_PL ) {
				$send_methods = [
					'courier' => __( 'Courier', EasyPack::$text_domain )
					,
					'pop'     => __( 'POP', EasyPack::$text_domain ),
				];
			} else {
				$send_methods = [
					'parcel_machine' => __( 'Parcel locker',
						EasyPack::$text_domain ),
				];
			}
			$selected_service
				= $shipment_service->get_customer_service_name_by_id(
				self::SERVICE_ID );
			include( 'views/html-order-metabox-courier-cod.php' );

			wp_nonce_field( self::NONCE_ACTION, 'wp_nonce' );
			if ( ! $output ) {
				$out = ob_get_clean();

				return $out;
			}
		}


		public static function ajax_get_payment_status() {

			$ret = [ 'status' => 'ok', 'message' => '' ];

			$paecel_id = $_POST['parcel_id'];

			$order_id = $_POST['order_id'];
			$order    = wc_get_order( $order_id );
			$post     = get_post( $order_id );

			$status = get_post_meta( $order_id, '_easypack_status', true );

			$easypack_parcels = get_post_meta( $order_id, '_easypack_parcels',
				true );
			$stickers         = [];

			if ( $easypack_parcels ) {
				foreach ( $easypack_parcels as $key => $parcel ) {
					try {
						if ( $parcel['easypack_data']['id'] == $parcel_id ) {
							$easypack_parcel
								= EasyPack_API()->customer_parcel( $parcel_id );
						}
					} catch ( Exception $e ) {
						$ret['status']  = 'error';
						$ret['message'] .= $e->getMessage();
					}
				}
			}

			if ( $ret['status'] == 'ok' ) {
			}
			echo json_encode( $ret );
			wp_die();
		}


	}
}
